@@ -8,6 +8,7 @@ use winit_input_helper::WinitInputHelper;
 const WIDTH: u32 = 640;
 const HEIGHT: u32 = 640;
 
+use funutd::math::*;
 use funutd::prelude::*;
 
 /// Application state.
@@ -82,31 +83,22 @@ impl World {
     ///
     /// Assumes the default texture format: `wgpu::TextureFormat::Rgba8UnormSrgb`
     fn draw(&self, frame: &mut [u8]) {
-        #[allow(unused_variables)]
-        let texture = palette(
-            Space::HSL,
-            0.6901543,
-            0.9025886,
-            posterize(
-                9.943014,
-                0.16815351,
-                rotate(
-                    8.86307,
-                    voronoi(2309937501, 31.571928, tile_all(), 7, 3, 2),
-                    voronoi(2538691872, 2.63204, tile_all(), 6, 5, 7),
-                ),
-            ),
-        );
-        /*
-        let texture = palette(Space::HSL, 0.50937665, 0.7222409, posterize(3.8965485, 0.60872394, softmix3(5.2831173, vnoise(1974317952, 10.774254, tile_all()), voronoi(1974803501, 24.273146, tile_all(), 5, 9, 7))));
-        */
         let mut dna = Dna::new(256, self.z as u64);
         let texture = genmap3palette(20.0, &mut dna);
 
-        if self.phase > 1.0 / 40.0 && self.phase < 1.0 / 24.0 {
+        if self.phase > 0.0 && self.phase < 1.0 / 24.0 {
             println!("{}", texture.get_code());
         }
 
+        //let texture = palette(Space::HSV, 0.0, 1.0, 0.0, 0.0, fractal(2.0, 8, 0.5, 2.0, 0.0, 0.0, noise_basis(1, tile_none())));
+        //let texture = palette(Space::HSV, 0.0, 1.0, 0.0, 1.0, fractal(2.0, 8, 0.5, 2.0, 0.0, 0.0, vnoise_basis(1, tile_none())));
+        //let texture = palette(Space::HSV, 0.0, 0.2, 0.0, 1.0, noise(1, 32.0, tile_none()));
+        //let texture = palette(Space::HSV, 0.192, 0.418, 0.015, 0.403, fractal(4.660271, 4, 0.76092863, 2.532766, 0.11465196, 0.0, voronoi_basis(3581209750, tile_all(), 12, 0, 6)));
+        //let texture = palette(Space::HSL, 0.739, 0.448, 0.281, 0.665, displace(0.18501252, voronoi(278220278, 17.42609, tile_all(), 0, 7, 10), voronoi(3737477767, 4.2395425, tile_all(), 2, 1, 0)));
+        //let texture = palette(Space::HSL, 0.406, 0.814, 0.483, 0.329, fractal(3.4102457, 7, 0.4548667, 2.789417, 0.0, 0.0, vnoise_basis(2316030952, tile_all())));
+        //let texture = palette(Space::HSV, 0.111, 0.712, 0.368, 0.051, displace(0.10156162, rotate(9.893959, noise(4137245708, 8.33033, tile_all()), voronoi(1284792858, 4.5874896, tile_all(), 7, 3, 3)), fractal(5.103115, 4, 0.47705963, 2.7184772, 0.0, 1.3609127, voronoi_basis(749463054, tile_all(), 5, 10, 2))));
+        //let texture = palette(Space::HSL, 0.296, 0.515, 0.212, 0.331, saturate(5.5411787, rotate(5.0376725, voronoi(1660873412, 25.0088, tile_all(), 11, 0, 11), noise(2384626526, 4.481734, tile_all()))));
+
         for (i, pixel) in frame.chunks_exact_mut(4).enumerate() {
             let x = (i % WIDTH as usize) as i16;
             let y = (i / WIDTH as usize) as i16;
