@@ -44,7 +44,7 @@ impl<H: Hasher> Texture for VNoise<H> {
                     // Pick number of cells as a rough approximation to a Poisson distribution.
                     let n = match hash & 7 {
                         0 | 1 | 2 | 3 => 1,
-                        5 | 6 => 2,
+                        4 | 5 | 6 => 2,
                         _ => 3,
                     };
                     // Offset points from cell corner to queried point.
@@ -121,11 +121,12 @@ impl<H: Hasher> Texture for Noise<H> {
                 let mut offset = Vec3a::new(dx as f32, dy as f32, 0.0) - basis.d;
                 for dz in -1..=1 {
                     let mut hash = self.hasher.hash_z(&basis, hxy, dz);
-                    // Pick number of cells as a rough approximation to a Poisson distribution.
+                    // Pick number of cells as a rough approximation to a Poisson(2) distribution.
                     let n = match hash & 7 {
-                        0 | 1 | 2 | 3 => 1,
-                        5 | 6 => 2,
-                        _ => 3,
+                        0 | 1 | 2 => 1,
+                        3 | 4 | 5 => 2,
+                        6 => 3,
+                        _ => 4,
                     };
                     // Offset points from cell corner to queried point.
                     offset = vec3a(offset.x, offset.y, dz as f32 - basis.d.z);
@@ -138,7 +139,7 @@ impl<H: Hasher> Texture for Noise<H> {
                         if distance2 < radius * radius {
                             let distance = sqrt(distance2) / radius;
                             let color = hash_11(hash);
-                            let gradient = hash_11(hash64b(hash));
+                            let gradient = hash_unit(hash64d(hash));
                             let blend = 1.0 - smooth5(distance);
                             result += color * blend * gradient.dot(p + offset);
                         }
@@ -149,7 +150,7 @@ impl<H: Hasher> Texture for Noise<H> {
                 }
             }
         }
-        result
+        result * 3.0
     }
 
     fn get_code(&self) -> String {
