@@ -2,11 +2,9 @@
 
 pub use super::color::*;
 pub use super::dna::*;
-pub use super::hash::*;
 pub use super::map3::*;
 pub use super::map3base::*;
 pub use super::map3gen::*;
-pub use super::math::*;
 pub use super::noise::*;
 pub use super::voronoi::*;
 pub use super::*;
